exe = "tester.exe"

toolchain = "msvc"

# optional
link_pool_depth = 1

# optional
builddir = {
    "gnu" :  "build"
  , "msvc" :  "build"
  , "clang" :  "build"
    }

includes = {
    "gnu" : [ "-I." ]
  , "msvc" : [ "/I." ]
  , "clang" : [ "-I." ]
    }

defines = {
    "gnu" : [ "-DEXAMPLE=1" ]
  , "msvc" : [ "/DEXAMPLE=1" ]
  , "clang" : [ "-DEXAMPLE=1" ]
    }

cflags = {
    "gnu" : [ "-O2", "-g" ]
  , "msvc" : [ "/O2" ]
  , "clang" : [ "-O2", "-g" ]
    }

cxxflags = {
    "gnu" : [ "-O2", "-g" ]
  , "msvc" : [ "/O2", "/W4", "/EHsc"]
  , "clang" : [ "-O2", "-g", "-fsanitize=address" ]
    }

ldflags = {
    "gnu" : [ ]
  , "msvc" : [ ]
  , "clang" : [ "-fsanitize=address" ]
    }

# optionsl
cxx_files = [ "tester.cc" ]
c_files = [ ]

# You can register your own toolchain through register_toolchain function
def register_toolchain(ninja):
    pass




#!/usr/bin/env python3
# Enum class for v3dheadertypes
""" AUTO-GENERATED from v3dheadertypes.csv """
# Generated at 2023-02-09 04:54:07

class v3dheadertypes:
    v3dheadertypes_canvasWidth=1
    # UINT  Canvas width

    v3dheadertypes_canvasHeight=2
    # UINT  Canvas heighot

    v3dheadertypes_absolute=3
    # BOOL  true: absolute size; false: scale to canvas

    v3dheadertypes_minBound=4
    # TRIPLE  Scene minimum bounding box corners

    v3dheadertypes_maxBound=5
    # TRIPLE  Scene maximum bounding box corners

    v3dheadertypes_orthographic=6
    # BOOL  true: orthographic; false: perspective

    v3dheadertypes_angleOfView=7
    # REAL  Field of view angle (in radians)

    v3dheadertypes_initialZoom=8
    # REAL  Initial zoom

    v3dheadertypes_viewportShift=9
    # PAIR  Viewport shift (for perspective projection)

    v3dheadertypes_viewportMargin=10
    # PAIR  Margin around viewport

    v3dheadertypes_light=11
    # RGB  Direction and color of each point light source

    v3dheadertypes_background=12
    # RGBA  Background color

    v3dheadertypes_zoomFactor=13
    # REAL  Zoom base factor

    v3dheadertypes_zoomPinchFactor=14
    # REAL  Zoom pinch factor

    v3dheadertypes_zoomPinchCap=15
    # REAL  Zoom pinch limit

    v3dheadertypes_zoomStep=16
    # REAL  Zoom power step

    v3dheadertypes_shiftHoldDistance=17
    # REAL  Shift-mode maximum hold distance (pixels)

    v3dheadertypes_shiftWaitTime=18
    # REAL  Shift-mode hold time (milliseconds)

    v3dheadertypes_vibrateTime=19
    # REAL  Shift-mode vibrate time (milliseconds)

# End of File



exe = "test_tinyexr.exe"

# "gnu" or "msvc" are provided as predefined toolchain
toolchain = "msvc"

# optional
link_pool_depth = 1

# optional
builddir = {
    "gnu" :  "build"
  , "msvc" :  "build"
  , "clang" :  "build"
    }

# required
includes = {
    "gnu" : [ "-I." ]
  , "msvc" : [ "/I." ]
  , "clang" : [ "-I." ]
    }

# required
defines = {
    "gnu" : [ "-DEXAMPLE=1" ]
  , "msvc" : [ "/DEXAMPLE=1" ]
  , "clang" : [ "-DEXAMPLE=1" ]
    }

# required
cflags = {
    "gnu" : [ "-O2", "-g" ]
  , "msvc" : [ "/O2" ]
  , "clang" : [ "-O2", "-g" ]
    }

# required
cxxflags = {
    "gnu" : [ "-O2", "-g" ]
  , "msvc" : [ "/O2", "/W4" ]
  , "clang" : [ "-O2", "-g", "-fsanitize=address" ]
    }

# required
ldflags = {
    "gnu" : [ ]
  , "msvc" : [ ]
  , "clang" : [ "-fsanitize=address" ]
    }

# optionsl
cxx_files = [ "test_tinyexr.cc" ]
c_files = [ ]

# You can register your own toolchain through register_toolchain function
def register_toolchain(ninja):
    pass

    #ninja.rule('clangcxx', description='CXX $out',
    #    command='$clangcxx -MMD -MF $out.d $clangdefines $clangincludes $clangcxxflags -c $in -o $out',
    #    depfile='$out.d', deps='gcc')
    #ninja.rule('clangcc', description='CC $out',
    #    command='$clangcc -MMD -MF $out.d $clangdefines $clangincludes $clangcflags -c $in -o $out',
    #    depfile='$out.d', deps='gcc')
    #ninja.rule('clanglink', description='LINK $out', pool='link_pool',
    #    command='$clangld $clangldflags -o $out $in $libs')
    #ninja.rule('clangar', description='AR $out', pool='link_pool',
    #    command='$clangar rsc $out $in')
    #ninja.rule('clangstamp', description='STAMP $out', command='touch $out')
    #ninja.newline()

    #ninja.variable('clangcxx', 'clang++')
    #ninja.variable('clangcc', 'clang')
    #ninja.variable('clangld', 'clang++')
    #ninja.variable('clangar', 'ar')
    #ninja.newline()





exe = "tester"

# "gnu" or "clang"
toolchain = "gnu"

# optional
link_pool_depth = 1

# optional
builddir = {
    "gnu" :  "build"
  , "msvc" :  "build"
  , "clang" :  "build"
    }

includes = {
    "gnu" : [ "-I." ]
  , "msvc" : [ "/I." ]
  , "clang" : [ "-I." ]
    }

defines = {
    "gnu" : [ ]
  , "msvc" : [ ]
  , "clang" : [  ]
    }

cflags = {
    "gnu" : [ "-O2", "-g" ]
  , "msvc" : [ "/O2" ]
  , "clang" : [ "-O2", "-g" ]
    }

# Warn as much as possible: http://qiita.com/MitsutakaTakeda/items/6b9966f890cc9b944d75
cxxflags = {
    "gnu" : [ "-O2", "-g", "-pedantic -Wall -Wextra -Wcast-align -Wcast-qual -Wctor-dtor-privacy -Wdisabled-optimization -Wformat=2 -Winit-self -Wmissing-declarations -Wmissing-include-dirs -Wold-style-cast -Woverloaded-virtual -Wredundant-decls -Wshadow -Wsign-conversion -Wsign-promo -Wstrict-overflow=5 -Wswitch-default -Wundef -Werror -Wno-unused", "-fsanitize=address" ]
  , "msvc" : [ "/O2", "/W4" ]
  , "clang" : [ "-O2", "-g", "-Werror -Weverything -Wno-c++98-compat -Wno-c++98-compat-pedantic", "-fsanitize=address" ]
    }

ldflags = {
    "gnu" : [ "-fsanitize=address" ]
  , "msvc" : [ ]
  , "clang" : [ "-fsanitize=address" ]
    }

cxx_files = [ "tester.cc" ]
c_files = [ ]

# You can register your own toolchain through register_toolchain function
def register_toolchain(ninja):
    pass




import OpenEXR
import Imath
import numpy as np
import simpleimageio as sio

width = 420
height = 32
border_left = 0
border_right = 420 - 80
num_splats = 10000

red = np.zeros((height, width), dtype=np.float32)
green = np.zeros((height, width), dtype=np.float32)
blue = np.zeros((height, width), dtype=np.float32)

# splat random color values
rng = np.random.default_rng()
row = rng.integers(low=0, high=height, size=num_splats)
col = rng.integers(low=border_left, high=border_right, size=num_splats)

# if any of the three channels has a fixed value, the problem goes away!
red[row, col] = rng.random(num_splats)
green[row, col] = rng.random(num_splats)
blue[row, col] = rng.random(num_splats)

# add a bunch of test pixels
red[-8, -10] = 1
green[-8, -10] = 1
blue[-8, -10] = 1

red[-4, -8] = 1
green[-4, -8] = 1
blue[-4, -8] = 1

red[-4, -2] = 1
green[-4, -2] = 1
blue[-4, -2] = 1

red[-2, -3] = 0 # setting this to anything other than 0 fixes the problem
green[-2, -3] = 1
blue[-2, -3] = 1

# fill in all of the black region with 0-red color
# red[:,border_right:] = 0
# green[:,border_right:] = 1
# blue[:,border_right:] = 1

# write PIZ compressed via OpenEXR
header = OpenEXR.Header(width, height)
header['compression'] = Imath.Compression(Imath.Compression.PIZ_COMPRESSION)
exr = OpenEXR.OutputFile("gen.exr", header)
exr.writePixels({'R': red.tobytes(), 'G': green.tobytes(), 'B': blue.tobytes()})
exr.close()

# read back in via tinyexr (used internally by simpleimageio)
tinyresult = sio.read("gen.exr")
sio.write("test2.exr", tinyresult)



#!/usr/bin/env python3
# Enum class for v3dtypes
""" AUTO-GENERATED from v3dtypes.csv """
# Generated at 2023-02-09 04:54:07

class v3dtypes:
    v3dtypes_material=1
    v3dtypes_transform=2
    v3dtypes_element=3
    v3dtypes_centers=4
    v3dtypes_header=5
    v3dtypes_line=64
    v3dtypes_triangle=65
    v3dtypes_quad=66
    v3dtypes_curve=128
    v3dtypes_bezierTriangle=129
    v3dtypes_bezierPatch=130
    v3dtypes_lineColor=192
    v3dtypes_triangleColor=193
    v3dtypes_quadColor=194
    v3dtypes_curveColor=256
    v3dtypes_bezierTriangleColor=257
    v3dtypes_bezierPatchColor=258
    v3dtypes_triangles=512
    v3dtypes_disk=1024
    v3dtypes_cylinder=1025
    v3dtypes_tube=1026
    v3dtypes_sphere=1027
    v3dtypes_halfSphere=1028
    v3dtypes_animation=2048
    v3dtypes_pixel=4096
# End of File



#!/usr/bin/env python3

# A script to generate enums in different languages from a CSV file.
# A CSV File contains
# enum1, 0
# enum2, ..
# ...
# enumn, n
# where 0,...,n are numbers.
#
# Written by Supakorn "Jamie" Rassameemasmuang <jamievlin@outlook.com>

from typing import List, Tuple, Any, Union
from datetime import datetime
import io
import argparse
import os
import sys
import time
import re

def cleanComment(s):
    return re.sub(r' *#',' ',s)

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('-language', '--language', type=str, required=True)
    parser.add_argument('-o', '--output', type=str, required=True)
    parser.add_argument('-i', '--input', type=str, required=True)
    parser.add_argument('-name', '--name', type=str, required=True)
    parser.add_argument('-xopt', '--xopt', type=str, nargs='*')
    return parser.parse_args()


def create_enums(filename: str) -> List[Union[Tuple[str, int, str], Tuple[str, int]]]:
    final_list=[]
    with io.open(filename,newline='') as rawfile:
        for line in rawfile.readlines():
            if line.startswith('#') or line.strip() == '':
                continue
            raw_line=line.strip().split(',')
            raw_str, raw_number=raw_line[0:2]
            comment=None
            if len(raw_line)>=3:
                comment=raw_line[-1]
                final_list.append((raw_str.strip(), int(raw_number.strip()),comment))
            else:
                final_list.append((raw_str.strip(), int(raw_number.strip())))
    return final_list


def datetime_now():
    return datetime.utcfromtimestamp(
        int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
    )


def generate_enum_cpp(outname, enums, name, comment=None, *args, **kwargs):
    with io.open(outname, 'w') as fil:
        fil.write('// Enum class for {0}\n'.format(name))
        if comment is not None:
            fil.write('// {0}\n'.format(comment))
        fil.write('// Generated at {0}\n\n'.format(datetime_now()))
        if 'namespace' in kwargs:
            fil.write('namespace {0}\n'.format(kwargs['namespace']))
            fil.write('{\n')

        fil.write('enum {0} : uint32_t\n'.format(name))
        fil.write('{\n')

        for enumTxt, enumNum, *ar in enums:
            fil.write('{0}={1},\n'.format(enumTxt, enumNum))
            if len(ar) > 0:
                comment=cleanComment(ar[-1])
                if comment is not None:
                    fil.write('// {0}\n\n'.format(comment.strip()))

        fil.write('};\n\n')

        if 'namespace' in kwargs:
            fil.write('}} // namespace {0}\n'.format(kwargs['namespace']))
        fil.write('// End of File\n')


def generate_enum_java(outname, enums, name, comment=None, *args, **kwargs):
    with io.open(outname, 'w') as fil:
        fil.write('// Enum class for {0}\n'.format(name))
        if comment is not None:
            fil.write('// {0}\n'.format(comment))
        fil.write('// Generated at {0}\n\n'.format(datetime_now()))

        if 'package' in kwargs:
            fil.write('package {0};\n'.format(kwargs['package']))
        fil.write('\n')

        fil.write('public enum {0} {{\n'.format(name))

        spaces = kwargs.get('spaces', 4)
        spaces_tab=' '*spaces

        for i in range(len(enums)):
            enumTxt, enumNum, *ar = enums[i]
            endsep=',' if i < len(enums) - 1 else ';'
            fil.write('{2}{0}({1}){3}\n'.format(enumTxt,enumNum,spaces_tab,endsep))
            if len(ar) > 0:
                comment=cleanComment(ar[-1])
                if comment is not None:
                    fil.write('// {0}\n\n'.format(comment.strip()))

        out_lines=[
            '',
            '{0}(int value) {{'.format(name),
            '{0}this.value=value;'.format(spaces_tab),
            '}',
            'public String toString() {',
            '{0}return Integer.toString(value);'.format(spaces_tab),
            '}',
            'private int value;']

        for line in out_lines:
            fil.write(spaces_tab)
            fil.write(line)
            fil.write('\n')
        fil.write('};\n\n')
        fil.write('// End of File\n')


def generate_enum_asy(outname, enums, name, comment=None, *args, **kwargs):
    with io.open(outname, 'w') as fil:
        fil.write('// Enum class for {0}\n'.format(name))
        if comment is not None:
            fil.write('// {0}\n'.format(comment))
        fil.write('// Generated at {0}\n\n'.format(datetime_now()))

        fil.write('struct {0}\n'.format(name))
        fil.write('{\n')

        for enumTxt, enumNum, *ar in enums:
            fil.write('  int {0}={1};\n'.format(enumTxt, enumNum))
            if len(ar) > 0:
                comment=cleanComment(ar[-1])
                if comment is not None:
                    fil.write('// {0}\n\n'.format(comment.strip()))
        fil.write('};\n\n')
        fil.write('{0} {0};'.format(name))

        fil.write('// End of File\n')


def generate_enum_py(outname, enums, name, comment=None, *args, **kwargs):
    with io.open(outname, 'w') as fil:
        fil.write('#!/usr/bin/env python3\n')
        fil.write('# Enum class for {0}\n'.format(name))
        if comment is not None:
            fil.write('""" {0} """\n'.format(comment))
        fil.write('# Generated at {0}\n\n'.format(datetime_now()))
        fil.write('class {0}:\n'.format(name))
        for enumTxt, enumNum, *ar in enums:
            fil.write('    {0}_{2}={1}\n'.format(name, enumNum, enumTxt))
            if len(ar) > 0:
                comment=cleanComment(ar[-1])
                if comment is not None:
                    fil.write('    # {0}\n\n'.format(comment.strip()))
        fil.write('# End of File\n')


def main():
    arg = parse_args()
    if arg.language in {'python', 'py'}:
        fn = generate_enum_py
    elif arg.language in {'cxx', 'c++', 'cpp'}:
        fn = generate_enum_cpp
    elif arg.language in {'asy', 'asymptote'}:
        fn = generate_enum_asy
    elif arg.language in {'java'}:
        fn = generate_enum_java
    else:
        return 1

    custom_args = dict()
    if arg.xopt is not None:
        for xopt in arg.xopt:
            key, val = xopt.split('=')
            custom_args[key] = val

    enums = create_enums(arg.input)
    fn(arg.output, enums, arg.name, 'AUTO-GENERATED from '+arg.input, **custom_args)


if __name__ == '__main__':
    sys.exit(main() or 0)
